@@ -5,7 +5,11 @@ pub mod consts;
 pub mod hostfxr;
 
 /// Module containing the raw bindings for nethost.
+#[cfg(feature = "nethost")]
 pub mod nethost;
 
 mod type_aliases;
 pub use type_aliases::*;
+
+mod status_code;
+pub use status_code::*;
