@@ -1,19 +1,14 @@
 use crate::{
-    bindings::{
-        char_t,
-        hostfxr::{hostfxr_handle, hostfxr_initialize_parameters, HostfxrLib},
+    bindings::hostfxr::{hostfxr_handle, hostfxr_initialize_parameters, HostfxrLib},
+    error::{HostingError, HostingResult},
+    hostfxr::{
+        HostfxrContext, HostfxrHandle, InitializedForCommandLine, InitializedForRuntimeConfig,
     },
-    nethost,
+    nethost::LoadHostfxrError,
     pdcstring::PdCStr,
-    Error,
 };
 use dlopen::wrapper::Container;
-use std::{ffi::OsStr, mem::MaybeUninit, ptr};
-
-use super::{
-    HostExitCode, HostfxrContext, HostfxrHandle, InitializedForCommandLine,
-    InitializedForRuntimeConfig,
-};
+use std::{convert::TryInto, ffi::OsStr, mem::MaybeUninit, ptr};
 
 /// A struct representing a loaded hostfxr library.
 pub struct Hostfxr {
@@ -25,15 +20,14 @@ impl !Send for Hostfxr {}
 
 impl Hostfxr {
     /// Loads the hostfxr library from the given path.
-    pub fn load_from_path(path: impl AsRef<OsStr>) -> Result<Self, Error> {
-        Ok(Self {
-            lib: unsafe { Container::load(path)? },
-        })
+    pub fn load_from_path(path: impl AsRef<OsStr>) -> Result<Self, dlopen::Error> {
+        unsafe { Container::load(path) }.map(|lib| Self { lib })
     }
 
-    /// Locates the hostfxr library using [`nethost`] and loads it.
-    pub fn load_with_nethost() -> Result<Self, Error> {
-        nethost::load_hostfxr()
+    /// Locates the hostfxr library using [`nethost`](crate::nethost) and loads it.
+    #[cfg(feature = "nethost")]
+    pub fn load_with_nethost() -> Result<Self, LoadHostfxrError> {
+        crate::nethost::load_hostfxr()
     }
 
     /// Initializes the hosting components for a dotnet command line running an application
@@ -44,7 +38,7 @@ impl Hostfxr {
     /// * Find the root framework (`Microsoft.NETCore.App`) and load the hostpolicy from it
     /// * The hostpolicy will then process all relevant `.deps.json` files and produce the list of assemblies, native search paths and other artifacts needed to initialize the runtime.
     ///
-    /// The functions will **NOT** load the CoreCLR runtime. They just prepare everything to the point where it can be loaded.
+    /// The functions will **NOT** load the `CoreCLR` runtime. They just prepare everything to the point where it can be loaded.
     ///
     /// # Arguments
     ///  * `app_path`:
@@ -57,7 +51,7 @@ impl Hostfxr {
     pub fn initialize_for_dotnet_command_line(
         &self,
         app_path: impl AsRef<PdCStr>,
-    ) -> Result<HostfxrContext<InitializedForCommandLine>, Error> {
+    ) -> Result<HostfxrContext<InitializedForCommandLine>, HostingError> {
         self.initialize_for_dotnet_command_line_with_args(&[app_path.as_ref()])
     }
 
@@ -69,7 +63,7 @@ impl Hostfxr {
     /// * Find the root framework (`Microsoft.NETCore.App`) and load the hostpolicy from it
     /// * The hostpolicy will then process all relevant `.deps.json` files and produce the list of assemblies, native search paths and other artifacts needed to initialize the runtime.
     ///
-    /// The functions will **NOT** load the CoreCLR runtime. They just prepare everything to the point where it can be loaded.
+    /// The functions will **NOT** load the `CoreCLR` runtime. They just prepare everything to the point where it can be loaded.
     ///
     /// # Arguments
     ///  * `app_path`:
@@ -77,7 +71,7 @@ impl Hostfxr {
     ///  * `host_path`:
     ///     Path to the native host (typically the `.exe`).
     ///     This value is not used for anything by the hosting components.
-    ///     It's just passed to the CoreCLR as the path to the executable.
+    ///     It's just passed to the `CoreCLR` as the path to the executable.
     ///     It can point to a file which is not executable itself, if such file doesn't exist (for example in COM activation scenarios this points to the `comhost.dll`).
     ///     This is used by PAL to initialize internal command line structures, process name and so on.
     ///
@@ -89,7 +83,7 @@ impl Hostfxr {
         &self,
         app_path: impl AsRef<PdCStr>,
         host_path: impl AsRef<PdCStr>,
-    ) -> Result<HostfxrContext<InitializedForCommandLine>, Error> {
+    ) -> Result<HostfxrContext<InitializedForCommandLine>, HostingError> {
         self.initialize_for_dotnet_command_line_with_args_and_host_path(
             &[app_path.as_ref()],
             host_path,
@@ -104,7 +98,7 @@ impl Hostfxr {
     /// * Find the root framework (`Microsoft.NETCore.App`) and load the hostpolicy from it
     /// * The hostpolicy will then process all relevant `.deps.json` files and produce the list of assemblies, native search paths and other artifacts needed to initialize the runtime.
     ///
-    /// The functions will **NOT** load the CoreCLR runtime. They just prepare everything to the point where it can be loaded.
+    /// The functions will **NOT** load the `CoreCLR` runtime. They just prepare everything to the point where it can be loaded.
     ///
     /// # Arguments
     ///  * `app_path`:
@@ -123,7 +117,7 @@ impl Hostfxr {
         &self,
         app_path: impl AsRef<PdCStr>,
         dotnet_root: impl AsRef<PdCStr>,
-    ) -> Result<HostfxrContext<InitializedForCommandLine>, Error> {
+    ) -> Result<HostfxrContext<InitializedForCommandLine>, HostingError> {
         self.initialize_for_dotnet_command_line_with_args_and_dotnet_root(
             &[app_path.as_ref()],
             dotnet_root,
@@ -138,7 +132,7 @@ impl Hostfxr {
     /// * Find the root framework (`Microsoft.NETCore.App`) and load the hostpolicy from it
     /// * The hostpolicy will then process all relevant `.deps.json` files and produce the list of assemblies, native search paths and other artifacts needed to initialize the runtime.
     ///
-    /// The functions will **NOT** load the CoreCLR runtime. They just prepare everything to the point where it can be loaded.
+    /// The functions will **NOT** load the `CoreCLR` runtime. They just prepare everything to the point where it can be loaded.
     ///
     /// # Arguments
     ///  * `args`:
@@ -153,7 +147,7 @@ impl Hostfxr {
     pub fn initialize_for_dotnet_command_line_with_args(
         &self,
         args: &[&PdCStr],
-    ) -> Result<HostfxrContext<InitializedForCommandLine>, Error> {
+    ) -> Result<HostfxrContext<InitializedForCommandLine>, HostingError> {
         unsafe {
             self.initialize_for_dotnet_command_line_with_parameters(args.as_ref(), ptr::null())
         }
@@ -167,7 +161,7 @@ impl Hostfxr {
     /// * Find the root framework (`Microsoft.NETCore.App`) and load the hostpolicy from it
     /// * The hostpolicy will then process all relevant `.deps.json` files and produce the list of assemblies, native search paths and other artifacts needed to initialize the runtime.
     ///
-    /// The functions will **NOT** load the CoreCLR runtime. They just prepare everything to the point where it can be loaded.
+    /// The functions will **NOT** load the `CoreCLR` runtime. They just prepare everything to the point where it can be loaded.
     ///
     /// # Arguments
     ///  * `args`:
@@ -177,7 +171,7 @@ impl Hostfxr {
     ///  * `host_path`:
     ///     Path to the native host (typically the `.exe`).
     ///     This value is not used for anything by the hosting components.
-    ///     It's just passed to the CoreCLR as the path to the executable.
+    ///     It's just passed to the `CoreCLR` as the path to the executable.
     ///     It can point to a file which is not executable itself, if such file doesn't exist (for example in COM activation scenarios this points to the `comhost.dll`).
     ///     This is used by PAL to initialize internal command line structures, process name and so on.
     ///
@@ -189,7 +183,7 @@ impl Hostfxr {
         &self,
         args: &[&PdCStr],
         host_path: impl AsRef<PdCStr>,
-    ) -> Result<HostfxrContext<InitializedForCommandLine>, Error> {
+    ) -> Result<HostfxrContext<InitializedForCommandLine>, HostingError> {
         let parameters = hostfxr_initialize_parameters::with_host_path(host_path.as_ref().as_ptr());
         unsafe { self.initialize_for_dotnet_command_line_with_parameters(args, &parameters) }
     }
@@ -202,7 +196,7 @@ impl Hostfxr {
     /// * Find the root framework (`Microsoft.NETCore.App`) and load the hostpolicy from it
     /// * The hostpolicy will then process all relevant `.deps.json` files and produce the list of assemblies, native search paths and other artifacts needed to initialize the runtime.
     ///
-    /// The functions will **NOT** load the CoreCLR runtime. They just prepare everything to the point where it can be loaded.
+    /// The functions will **NOT** load the `CoreCLR` runtime. They just prepare everything to the point where it can be loaded.
     ///
     /// # Arguments
     ///  * `args`:
@@ -223,7 +217,7 @@ impl Hostfxr {
         &self,
         args: &[&PdCStr],
         dotnet_root: impl AsRef<PdCStr>,
-    ) -> Result<HostfxrContext<InitializedForCommandLine>, Error> {
+    ) -> Result<HostfxrContext<InitializedForCommandLine>, HostingError> {
         let parameters =
             hostfxr_initialize_parameters::with_dotnet_root(dotnet_root.as_ref().as_ptr());
         unsafe { self.initialize_for_dotnet_command_line_with_parameters(args, &parameters) }
@@ -233,22 +227,26 @@ impl Hostfxr {
         &self,
         args: &[&PdCStr],
         parameters: *const hostfxr_initialize_parameters,
-    ) -> Result<HostfxrContext<InitializedForCommandLine>, Error> {
+    ) -> Result<HostfxrContext<InitializedForCommandLine>, HostingError> {
         let mut hostfxr_handle = MaybeUninit::<hostfxr_handle>::uninit();
 
-        let result = self.lib.hostfxr_initialize_for_dotnet_command_line(
-            args.len() as i32,
-            args.as_ptr() as *const *const char_t,
-            parameters,
-            hostfxr_handle.as_mut_ptr(),
-        );
+        let result = unsafe {
+            self.lib.hostfxr_initialize_for_dotnet_command_line(
+                args.len().try_into().unwrap(),
+                args.as_ptr().cast(),
+                parameters,
+                hostfxr_handle.as_mut_ptr(),
+            )
+        };
 
-        HostExitCode::from(result).to_result()?;
+        HostingResult::from(result).into_result()?;
 
-        Ok(HostfxrContext::new(
-            HostfxrHandle::new_unchecked(hostfxr_handle.assume_init()),
-            self,
-        ))
+        Ok(unsafe {
+            HostfxrContext::from_handle(
+                HostfxrHandle::new_unchecked(hostfxr_handle.assume_init()),
+                self,
+            )
+        })
     }
 
     /// This function loads the specified `.runtimeconfig.json`, resolve all frameworks, resolve all the assets from those frameworks and
@@ -261,7 +259,7 @@ impl Hostfxr {
     /// * Find the root framework (`Microsoft.NETCore.App`) and load the hostpolicy from it
     /// * The hostpolicy will then process all relevant `.deps.json` files and produce the list of assemblies, native search paths and other artifacts needed to initialize the runtime.
     ///
-    /// The functions will **NOT** load the CoreCLR runtime. They just prepare everything to the point where it can be loaded.
+    /// The functions will **NOT** load the `CoreCLR` runtime. They just prepare everything to the point where it can be loaded.
     ///
     /// # Arguments
     ///  * `runtime_config_path`:
@@ -272,7 +270,7 @@ impl Hostfxr {
     pub fn initialize_for_runtime_config(
         &self,
         runtime_config_path: impl AsRef<PdCStr>,
-    ) -> Result<HostfxrContext<InitializedForRuntimeConfig>, Error> {
+    ) -> Result<HostfxrContext<InitializedForRuntimeConfig>, HostingError> {
         unsafe {
             self.initialize_for_runtime_config_with_parameters(runtime_config_path, ptr::null())
         }
@@ -288,7 +286,7 @@ impl Hostfxr {
     /// * Find the root framework (`Microsoft.NETCore.App`) and load the hostpolicy from it
     /// * The hostpolicy will then process all relevant `.deps.json` files and produce the list of assemblies, native search paths and other artifacts needed to initialize the runtime.
     ///
-    /// The functions will **NOT** load the CoreCLR runtime. They just prepare everything to the point where it can be loaded.
+    /// The functions will **NOT** load the `CoreCLR` runtime. They just prepare everything to the point where it can be loaded.
     ///
     /// # Arguments
     ///  * `runtime_config_path`:
@@ -297,7 +295,7 @@ impl Hostfxr {
     ///  * `host_path`:
     ///     Path to the native host (typically the `.exe`).
     ///     This value is not used for anything by the hosting components.
-    ///     It's just passed to the CoreCLR as the path to the executable.
+    ///     It's just passed to the `CoreCLR` as the path to the executable.
     ///     It can point to a file which is not executable itself, if such file doesn't exist (for example in COM activation scenarios this points to the `comhost.dll`).
     ///     This is used by PAL to initialize internal command line structures, process name and so on.
     ///
@@ -306,7 +304,7 @@ impl Hostfxr {
         &self,
         runtime_config_path: impl AsRef<PdCStr>,
         host_path: impl AsRef<PdCStr>,
-    ) -> Result<HostfxrContext<InitializedForRuntimeConfig>, Error> {
+    ) -> Result<HostfxrContext<InitializedForRuntimeConfig>, HostingError> {
         let parameters = hostfxr_initialize_parameters::with_host_path(host_path.as_ref().as_ptr());
         unsafe {
             self.initialize_for_runtime_config_with_parameters(runtime_config_path, &parameters)
@@ -322,7 +320,7 @@ impl Hostfxr {
     /// * Find the root framework (`Microsoft.NETCore.App`) and load the hostpolicy from it
     /// * The hostpolicy will then process all relevant `.deps.json` files and produce the list of assemblies, native search paths and other artifacts needed to initialize the runtime.
     ///
-    /// The functions will **NOT** load the CoreCLR runtime. They just prepare everything to the point where it can be loaded.
+    /// The functions will **NOT** load the `CoreCLR` runtime. They just prepare everything to the point where it can be loaded.
     ///
     /// # Arguments
     ///  * `runtime_config_path`:
@@ -339,7 +337,7 @@ impl Hostfxr {
         &self,
         runtime_config_path: impl AsRef<PdCStr>,
         dotnet_root: impl AsRef<PdCStr>,
-    ) -> Result<HostfxrContext<InitializedForRuntimeConfig>, Error> {
+    ) -> Result<HostfxrContext<InitializedForRuntimeConfig>, HostingError> {
         let parameters =
             hostfxr_initialize_parameters::with_dotnet_root(dotnet_root.as_ref().as_ptr());
         unsafe {
@@ -351,20 +349,24 @@ impl Hostfxr {
         &self,
         runtime_config_path: impl AsRef<PdCStr>,
         parameters: *const hostfxr_initialize_parameters,
-    ) -> Result<HostfxrContext<InitializedForRuntimeConfig>, Error> {
+    ) -> Result<HostfxrContext<InitializedForRuntimeConfig>, HostingError> {
         let mut hostfxr_handle = MaybeUninit::uninit();
 
-        let result = self.lib.hostfxr_initialize_for_runtime_config(
-            runtime_config_path.as_ref().as_ptr(),
-            parameters,
-            hostfxr_handle.as_mut_ptr(),
-        );
+        let result = unsafe {
+            self.lib.hostfxr_initialize_for_runtime_config(
+                runtime_config_path.as_ref().as_ptr(),
+                parameters,
+                hostfxr_handle.as_mut_ptr(),
+            )
+        };
 
-        HostExitCode::from(result).to_result()?;
+        HostingResult::from(result).into_result()?;
 
-        Ok(HostfxrContext::new(
-            HostfxrHandle::new_unchecked(hostfxr_handle.assume_init()),
-            self,
-        ))
+        Ok(unsafe {
+            HostfxrContext::from_handle(
+                HostfxrHandle::new_unchecked(hostfxr_handle.assume_init()),
+                self,
+            )
+        })
     }
 }
