@@ -4,8 +4,5 @@ pub use library::*;
 mod context;
 pub use context::*;
 
-mod host_exit_code;
-pub use host_exit_code::*;
-
 mod delegate_loader;
 pub use delegate_loader::*;
