@@ -8,7 +8,7 @@ use std::{
 
 use super::{NulError, PdCStrInner, PdCStringInner, PdUChar};
 
-/// A platform-dependent c-like string type for interacting with the hostfxr API.
+/// A platform-dependent c-like string type for interacting with the `hostfxr` and `nethost` API.
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Hash, Clone)]
 #[repr(transparent)]
 pub struct PdCString(pub(crate) PdCStringInner);
@@ -22,7 +22,7 @@ impl PdCString {
     }
 }
 
-/// A borrowed slice of a PdCString.
+/// A borrowed slice of a [`PdCString`].
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
 #[repr(transparent)]
 pub struct PdCStr(pub(crate) PdCStrInner);
@@ -68,11 +68,11 @@ impl DerefMut for PdCString {
 
 impl Default for PdCString {
     fn default() -> Self {
-        Self(Default::default())
+        Self(PdCStringInner::default())
     }
 }
 
-impl Display for PdCString {
+impl Display for PdCStr {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         self.0.fmt(f)
     }
@@ -90,6 +90,18 @@ impl From<PdCString> for PdCStringInner {
     }
 }
 
+impl<'a> From<&'a PdCStrInner> for &'a PdCStr {
+    fn from(s: &'a PdCStrInner) -> Self {
+        PdCStr::from_inner(s)
+    }
+}
+
+impl<'a> From<&'a PdCStr> for &'a PdCStrInner {
+    fn from(s: &'a PdCStr) -> Self {
+        s.to_inner()
+    }
+}
+
 impl<'a> TryFrom<&'a str> for PdCString {
     type Error = NulError;
 
